/*****************************************************
Interrupter for DRSSTCs. Two potentiometers used to set
on time and frequency. One button is used to start
and stop the interrupter, the other to change mode.
Timer0 is used to refresh the LCD at a set interval.
Timer1 is used to generate the interrupter waveform.

Data Types:

int8_t - Signed Char 
uint16_t - Unsigned Int 
uint32_t - Unsigned Long 
int64_t - Signed Long Long

(signed/unsigned) char - 1 byte 
(signed/unsigned) short - 2 bytes 
(signed/unsigned) int - 2 bytes 
(signed/unsigned) long - 4 bytes 
(signed/unsigned) long long - 8 bytes 
float - 4 bytes (floating point) 
double - alias to float


Project : 	SSTC Interrupter
AVR Type:	ATmega8
AVR Clock frequency: 8,00 MHz
*****************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>
#include "lcd.h"
#include "interrupterLib.h"

/***************************************************************************/

// Interrupter frequency, deciHz
#define minFreq 160
#define maxFreq 2250

// Interrupter ON time, s
#define minON 10
#define maxON 350

/***************************************************************************/

#define LCDUPDATEFREQUENCY 20
#define DEBOUNCETIMEOUT 15
#define BUTTON0 (!(PINC & (1 << PC2)))
#define BUTTON1 (!(PINC & (1 << PC3)))
#define CLOCKFREQUENCY (F_CPU/8.0)

/***************************************************************************/

// Declare global variables
volatile uint8_t LCDupdateTimer, timeout;
volatile uint16_t ICR1buffer;

// Overflow interrupt for Timer 0, used to update LCD
ISR (TIMER0_OVF_vect)
{
	LCDupdateTimer++;
	if (timeout != 0) timeout--;
}

ISR(TIMER1_COMPA_vect)
{
	// Set output pin low
	PORTD &=~(1 << PD1);
}

ISR(TIMER1_OVF_vect)
{
	// Set output pin high
	PORTD |= (1 << PD1);
	
	// Update frequency from buffer
	ICR1 = ICR1buffer;
}

void setInterrupter(double ontime, double frequency)
{
	// OCR1A is double buffered in hardware, ie only changed when the TOP value (TIMER1 OVF) is reached.
	double temp = ontime * CLOCKFREQUENCY / 1000000;
	OCR1A = temp;
	
	// ICR1 must be buffered in software
	temp = (10 * CLOCKFREQUENCY / frequency) + 1;
	ICR1buffer = temp;
}

void startInterrupter(void)
{
	// Prepare the interrupter
	TCNT1 = 0;
	PORTD |= (1 << PD1);
	// Connect timer 1 to 1:8 prescaler
	TCCR1B |= (0<<CS12)|(1<<CS11)|(0<<CS10);
	PORTB |= (1<<PB0);
}

void stopInterrupter(void)
{
	// Remove clock source, thus stopping timer 1
	TCCR1B &= 0xF8;
	// Set output low
	PORTD &=~(1 << PD1);
	PORTB &=~(1<<PB0);
}

// Main
int main(void)
{
// Declare local variables
uint8_t button1mode;
char line_txt[16];
double potentio1, potentio2;

// Initialize interrupter hardware
initInterrupterHardware("DRSSTC", "Interrupter");
button1mode = 0;
stopInterrupter();

while (1)
	{

	if (timeout == 0) {
		// Check if switch is pressed
		if (BUTTON0) {
			// Set debouce timeout on switch
			timeout = DEBOUNCETIMEOUT;
			
			// Start the interrupter sequence
			if (button1mode)
			{
				// Turn off the interrupter
				startInterrupter();
				button1mode = 0;
			}else
			{
				// Turn on the interrupter
				stopInterrupter();
				button1mode = 1;
			}
			
		}else if (BUTTON1) {
			// Set debouce timeout on switch
			timeout = DEBOUNCETIMEOUT;
		
			// Change interrupter operation mode
			
		}
	}
	
	// Get updated potentiometer values and write to LCD
	if (LCDupdateTimer >= LCDUPDATEFREQUENCY)
		{
		
		// Get potentiometer values as fraction of unity
		// Use logarithmic scale
		potentio1 = getPotentiometerValue(0);
		//potentio1 = (potentio1/255);
		potentio1 = (potentio1*potentio1/65025);
		potentio2 = getPotentiometerValue(1);
		//potentio2 = (potentio2/255);
		potentio2 = (potentio2*potentio2/65025);
		
		// Determine the ON-time
		potentio1 = minON + potentio1 * maxON;
		// Determine the frequency
		potentio2 = minFreq + potentio2 * maxFreq;
		
		// Use these values to set the interrupter
		setInterrupter(potentio1, potentio2);
		
		// Write the values to LCD
		sprintf(line_txt, "ON-time %u us", (uint16_t)potentio1);
		lcd_clrscr();
		lcd_puts(line_txt);
		
		lcd_gotoxy(0,1);
		sprintf(line_txt, "Freq %u Hz", (uint16_t)potentio2/10);
		lcd_puts(line_txt);
		LCDupdateTimer = 0;
		}
	}
	
return 0;
}
