/*****************************************************
Interrupter for SSTCs. Two potentiometers used to set
duty cycle and frequency. One button is used to start
the interrupter, the other to stop it. Timer0 is used
to refresh the LCD at a set interval. Timer1 is used
to generate the interrupter waveform.

Data Types:

int8_t - Signed Char 
uint16_t - Unsigned Int 
uint32_t - Unsigned Long 
int64_t - Signed Long Long

(signed/unsigned) char - 1 byte 
(signed/unsigned) short - 2 bytes 
(signed/unsigned) int - 2 bytes 
(signed/unsigned) long - 4 bytes 
(signed/unsigned) long long - 8 bytes 
float - 4 bytes (floating point) 
double - alias to float


Project : 	SSTC Interrupter
AVR Type:	ATmega8
AVR Clock frequency: 8,00 MHz
*****************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>
#include "lcd.h"
#include "interrupterLib.h"

/***************************************************************************/

#define minFreq 20			// Interrupter frequency
#define maxFreq 500

/***************************************************************************/

#define LCDupdateFrequency 20
#define debounceTimeout 15
#define Button0 (!(PINC & (1 << PC2)))
#define Button1 (!(PINC & (1 << PC3)))
#define ClockFrequency (F_CPU/8.0)

// Declare global variables
volatile uint16_t onPeriodOCR1, offPeriodOCR1;
volatile uint8_t LCDupdateTimer, timeout, pwmState, interrupterIsActive;

// Overflow interrupt for Timer 0, used to update LCD
ISR (TIMER0_OVF_vect) {
	LCDupdateTimer++;
	if (timeout != 0) timeout--;
}

ISR(TIMER1_COMPA_vect){

// Shift between on and off
	if (pwmState == 0) {
		PORTD &=~(1 << PD1);
		OCR1A = offPeriodOCR1;
		pwmState = 1;
	}else{
		PORTD |= (1 << PD1);
		OCR1A = onPeriodOCR1;
		pwmState = 0;
	}
}

uint8_t setInterrupter(double dutyratio, double frequency) {
	double temp = frequency;
	
	frequency = (1.0 / temp);
	frequency *= ClockFrequency;
	temp = dutyratio * frequency;
	onPeriodOCR1 = temp;
	offPeriodOCR1 = frequency - onPeriodOCR1;
	
	if (onPeriodOCR1 == 0) {
		return 0;
	}else if (offPeriodOCR1 == 0) {
		return 1;
	}else {
		return 3;
	}
}

void startInterrupter(void) {
	// Prepare the interrupter
	TCNT1 = 0;
	PORTD |= (1 << PD1);
	pwmState = 0;
	// Connect timer 1 to 1:8 prescaler
	TCCR1B |= (1 << CS11);
	interrupterIsActive = 1;
	PORTB |= (1<<PB0);
}

void stopInterrupter(void) {
	// Remove clock source, thus stopping timer 1
	TCCR1B &= 0xF8;
	// Set output low
	PORTD &=~(1 << PD1);
	interrupterIsActive = 0;
	PORTB &=~(1<<PB0);
}

// Main
void main(void) {

// Declare local variables
uint16_t dutyCycle, frequency;
uint8_t cycleTest = 0, shouldInterrupterBeActive;
char line_txt[16];
double potentio1, potentio2;

// Initialize interrupter hardware
initInterrupterHardware("SSTC", "Interrupter");
OCR1A = 10000;
shouldInterrupterBeActive = 0;
stopInterrupter();

while (1) {

	if (timeout == 0) {
		// Check if switch is pressed
		if (Button0) {
			// Set debouce timeout on switch
			timeout = debounceTimeout;
			
			// Start the interrupter sequence
			shouldInterrupterBeActive = 1;
			
			// Only start the interrupter if the duty cycle isn't zero or 100%
			if ((cycleTest == 3)) {
				startInterrupter();
			}
			
		}else if (Button1) {
			// Set debouce timeout on switch
			timeout = debounceTimeout;
		
			// Stop the interrupter sequence
			stopInterrupter();
			shouldInterrupterBeActive = 0;
		}
	}
	
	// Change values and write LCD
	if (LCDupdateTimer >= LCDupdateFrequency) {
		
		// Get potentiometer values as fraction of unity
		potentio1 = getPotentiometerValue(0);
		potentio1 = (potentio1/255);
		potentio2 = getPotentiometerValue(1);
		potentio2 = (potentio2/255);
		
		// Use these values to determine the frequency
		potentio2 = minFreq + potentio2 * maxFreq;
		
		// Use these values to set the interrupter
		cycleTest = setInterrupter(potentio1, potentio2);
		
		if (shouldInterrupterBeActive != 0)  {
			switch (cycleTest)
			{
			case (0): {
				// Set the output low and turn off the interrupter
				stopInterrupter();
				interrupterIsActive = 3;
			}break;
		
		    case (1): {
				// Set the output to constant high after turning off the interrupter
				stopInterrupter();
				interrupterIsActive = 3;
				PORTD |= (1 << PD1);
				PORTB |= (1 << PB0);
			}break;
			
		    default: {
				// If the interrupter is paused, then start it, as the duty cycle has changed
				if (interrupterIsActive == 3) startInterrupter();
			}break;
			}
		}
		
		// Write the values to LCD
		dutyCycle = potentio1 * 100;
		frequency = potentio2;
		
		sprintf(line_txt, "Duty Cycle %u %%",dutyCycle);
		lcd_clrscr();
		lcd_puts(line_txt);
		
		lcd_gotoxy(0,1);
		sprintf(line_txt, "Freq %u Hz",frequency);
		lcd_puts(line_txt);
		LCDupdateTimer = 0;
	}
}

}
