/*****************************************************
General purpose timer. One potentiometer is used to set
the delay before starting the timer, and the other sets
the actual timer. One button is used to start a count-
down, the other to cancel. Timer0 is used to refresh the
LCD at a set interval. Timer1 is used to generate the
timer clock.

Data Types:

int8_t - Signed Char 
uint16_t - Unsigned Int 
uint32_t - Unsigned Long 
int64_t - Signed Long Long

(signed/unsigned) char - 1 byte 
(signed/unsigned) short - 2 bytes 
(signed/unsigned) int - 2 bytes 
(signed/unsigned) long - 4 bytes 
(signed/unsigned) long long - 8 bytes 
float - 4 bytes (floating point) 
double - alias to float


Project : 	General purpose timer
AVR Type:	ATmega8
AVR Clock frequency: 8,00 MHz
*****************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>
#include "lcd.h"
#include "interrupterLib.h"

/***************************************************************************/

#define minTime 200					// Set timer limits in milliseconds
#define maxTime (65535 - minTime)

/***************************************************************************/

#define LCDupdateFrequency 245
#define debounceTimeout 15
#define Button0 (!(PINC & (1 << PC2)))
#define Button1 (!(PINC & (1 << PC3)))
#define ClockFrequency (F_CPU/1.0)

// Declare global variables
volatile uint16_t delayTime = 0, timerTime = 0;
volatile uint8_t LCDupdateTimer, timeout;

// Overflow interrupt for Timer 0, used to update LCD
ISR (TIMER0_OVF_vect) {
	LCDupdateTimer++;
	if (timeout != 0) timeout--;
}

void startTimer(void) {
	// Prepare the interrupter
	TCNT1 = 0;
	// Set output low
	PORTD &=~(1 << PD1);
	// Connect timer 1 to 1:1 prescaler
	TCCR1B |= (1 << CS10);
	PORTB |= (1<<PB0);
}

void stopTimer(void) {
	// Remove clock source, thus stopping timer 1
	TCCR1B &= 0xF8;
	// Set output low
	PORTD &=~(1 << PD1);
	PORTB &=~(1<<PB0);
}

ISR(TIMER1_COMPA_vect){
// Timer 1 overflow will be 1 millisecond clock
	if (delayTime == 0) {
		if (timerTime != 0) {
			timerTime--;
		}else {
			stopTimer();
		}
	}else {
		delayTime--;
	}
}

// Main
void main(void) {

// Declare local variables
uint8_t timerIsActive = 0;
char line_txt[16];
double potentio1, potentio2;

// Initialize interrupter hardware
initInterrupterHardware("General Purpose", "Timer unit");
OCR1A = 8000;
stopTimer();

while (1) {

// Check button states
	if (timeout == 0) {
		// Check if switch is pressed
		if (Button0 && (timerIsActive != 1)) {
			// Set debouce timeout on switch
			timeout = debounceTimeout;
			
			// Start the interrupter sequence
			startTimer();
			timerIsActive = 1;
			
		}else if (Button1) {
			// Set debouce timeout on switch
			timeout = debounceTimeout;
		
			// Stop the interrupter sequence
			stopTimer();
			timerIsActive = 0;
		}
	}
	
// Check timer state
	if (timerIsActive == 1) {
		// Preform timer functions
		
		// Check if output should be high
		if (delayTime == 0) {
			PORTD |= (1 << PD1);
		}
		
		// Check if timer has counted down
		if ((delayTime == 0) && (timerTime == 0)) {
			timerIsActive = 0;
			stopTimer();
		}
		
	}else {
		// Set timer values
		if (LCDupdateTimer >= LCDupdateFrequency) {
		
		// Get potentiometer values as fraction of unity
		potentio1 = getPotentiometerValue(0);
		potentio1 = (potentio1/255);
		potentio1 *= potentio1;
		potentio2 = getPotentiometerValue(1);
		potentio2 = (potentio2/255);
		potentio2 *= potentio2;
		
		// Use these values to determine the timer values
		// Delay timer can be set to zero, timer timer cannot
		// To ease setting low timer values, a logarithmic taper is used
		
		potentio1 *= (minTime + maxTime);
		potentio2 = minTime + potentio2 * maxTime;
		
		delayTime = potentio1;
		timerTime = potentio2;
		}
	}
	
	// Update times displayed on LCD
	if (LCDupdateTimer >= LCDupdateFrequency) {
		// Write the values to LCD
		if (delayTime < 10000) {
			sprintf(line_txt, "Delay %u ms",delayTime);
		}else {
			sprintf(line_txt, "Delay %u s",(delayTime/1000));
		
		}
	
		lcd_clrscr();
		lcd_puts(line_txt);
		lcd_gotoxy(0,1);
	
		if (timerTime < 10000) {
			sprintf(line_txt, "Timer %u ms",timerTime);
		}else {
			sprintf(line_txt, "Timer %u s",(timerTime/1000));
		}
	
		lcd_puts(line_txt);
		LCDupdateTimer = 0;
	}
}

}
